function [stoicMat, rxnTable,speciesList,speciesExtList,notesTable] = sbmlREAD(fileName)
%%%sbmlREAD%%
%function [stoicMat, rxnTable, speciesList] = sbmlREAD(fileName);
%to read metabolic model in SMBL file format into MATLAB
%generates stoichiometric matrix, reaction list and species list
%author Lake-Ee Quek, The University of Queensland, AIBN
%02-Oct-2008
%stoicMat: stoichiometric matrix
%rxnTable: [rxnID rxnEquation(short) rxnEquation(long) reversibility]
%speciesList: [species(name) species(ID)]
%fileName: sbml file name
% fileName = 'AraGEM vs1.0.1.sbml';
xDoc=xmlread(fileName);
lvl1 = xDoc.getElementsByTagName('model').item(0);
lvl2Rxn = lvl1.getElementsByTagName('listOfReactions').item(0);


lvl2Spe = lvl1.getElementsByTagName('listOfSpecies').item(0);
speShort={};
speLong={};
speExt=[];
for i = 0:lvl2Spe.getLength-1
    if lvl2Spe.item(i).getNodeType == lvl2Spe.item(i).ELEMENT_NODE
        speShort{end+1, 1} = char(lvl2Spe.item(i).getAttribute('id'));
        speLong{end+1, 1} = char(lvl2Spe.item(i).getAttribute('name'));
        speBC = char(lvl2Spe.item(i).getAttribute('boundaryCondition'));
        if strcmp(speBC,'true')
            speExt = [speExt; 1];
        else
            speExt = [speExt; 0];
        end
    end
end
stoicMat=[];
rxnTable=[];
notesTable=[];

for i = 0:lvl2Rxn.getLength-1
    if lvl2Rxn.item(i).getNodeType == lvl2Rxn.item(i).ELEMENT_NODE
        stoicLine = zeros(1,length(speExt));
        
        rxnID = char(lvl2Rxn.item(i).getAttribute('id'));
        rxnTable{end+1,1}=rxnID;
        rxnRev = char(lvl2Rxn.item(i).getAttribute('reversible'));
        if strcmp(rxnRev, 'false')
            rxnTable{end,4}=0;
        else
            rxnTable{end,4}=1;
        end
        lvl3Reactants = lvl2Rxn.item(i).getElementsByTagName('listOfReactants').item(0);
        lvl3Products= lvl2Rxn.item(i).getElementsByTagName('listOfProducts').item(0);
        lvl3Notes = lvl2Rxn.item(i).getElementsByTagName('annotation').item(0);
        rxnNotes='';
%        for j = 0:lvl3Notes.getLength-1
 %               if length(rxnNotes) == 0
  %                  rxnNotes=char(lvl3Notes.item(j).getTextContent);
   %             else
    %                rxnNotes=strcat([rxnNotes,'; ',char(lvl3Notes.item(j).getTextContent)]);
     %           end
      %      end
       % end
        
        rxnEQlong='';
        rxnEQshort='';
        for j = 0:lvl3Reactants.getLength-1
            if lvl3Reactants.item(j).getNodeType == lvl3Reactants.item(j).ELEMENT_NODE
                lvl4Spe = lvl3Reactants.item(j);
                species = char(lvl4Spe.getAttribute('species'));
                stoic = char(lvl4Spe.getAttribute('stoichiometry'));
                if length(stoic)==0
                    stoic = '1';
                end
                for k = 1:length(speExt)
                    if strcmp(species,speShort{k})
                        break
                    end
                end
                stoicLine(k) = -str2num(stoic);
                if length(rxnEQlong) == 0
                    rxnEQshort = strcat([stoic,' ', species]);
                    rxnEQlong = strcat([stoic,' ', speLong{k}]);
                else
                    rxnEQshort = strcat([rxnEQshort,' + ', stoic, ' ', species]);
                    rxnEQlong = strcat([rxnEQlong,' + ', stoic, ' ', speLong{k}]);
                end
            end
        end
        
        rxnEQshort = strcat([rxnEQshort, ' =']);
        rxnEQlong = strcat([rxnEQlong, ' =']);

        for j = 0:lvl3Products.getLength-1
            if lvl3Products.item(j).getNodeType == lvl3Products.item(j).ELEMENT_NODE
                lvl4Spe = lvl3Products.item(j);
                species = char(lvl4Spe.getAttribute('species'));
                stoic = char(lvl4Spe.getAttribute('stoichiometry'));
                if length(stoic)==0
                    stoic = '1';
                end
                for k = 1:length(speExt)
                    if strcmp(species,speShort{k})
                        break
                    end
                end
                stoicLine(k) = str2num(stoic);
                if rxnEQlong(end) == '='
                    rxnEQshort = strcat([rxnEQshort, ' ', stoic,' ', species]);
                    rxnEQlong = strcat([rxnEQlong, ' ', stoic,' ', speLong{k}]);
                else
                    rxnEQshort = strcat([rxnEQshort,' + ', stoic, ' ', species]);
                    rxnEQlong = strcat([rxnEQlong,' + ', stoic, ' ', speLong{k}]);
                end
            end
        end
        rxnTable{end,2}=regexprep(rxnEQshort,'  ',' ');
        rxnTable{end,3}=regexprep(rxnEQlong,'  ',' ');
        notesTable{end+1,1}=rxnNotes;
        stoicMat = [stoicMat;stoicLine];
    end
end

speciesList=[speLong speShort];
speciesExtList = speciesList(find(speExt==1),:);

