%clear all;clc;
load initData

stoicMat = sparse(stoicMat);

disp('original model...');
fprintf('no of mets:\t %1.0f\n',size(stoicMat,2));
fprintf('no of rxns:\t %1.0f\n',size(stoicMat,1));
fprintf('\n');

%%%%%duplicate generic model into tissue blocks%%%%%%
rxnLib_ori = rxnTable(:,1);
reversible = cell2mat(rxnTable(:,4));

bValue = 10000;
ub_ori = ones(size(rxnTable,1),1)*bValue;
lb_ori = zeros(size(rxnTable,1),1);
lb_ori(find(reversible==1)) = -bValue;

stoicBlock = []; rxnBlock = {}; speciesBlock = {};
lbBlock = [];  ubBlock = [];
disp('tissue block addition...');
for i = 1:size(compartment,1)
    printCell(compartment(i,:));
    if i == 1
        stoicBlock = stoicMat;
        rxnBlock(:,1) = rxnLib_ori;
        rxnBlock(:,2) = compartment(i,3);
        rxnBlock(:,3) = compartment(i,4);
        rxnBlock(:,4) = {'INT'};
        
        speciesBlock(:,1) = speciesList(:,2);
        speciesBlock(:,2) = speciesList(:,1);
        speciesBlock(:,3) = compartment(i,3);
        speciesBlock(:,4) = compartment(i,4);
        speciesBlock(:,5) = {'INT'};
        
        lbBlock = lb_ori;
        ubBlock = ub_ori;
        
    else
        currentRxnLength = size(rxnBlock,1);
        
        rxnBlock(currentRxnLength+1:currentRxnLength+length(rxnLib_ori),1) = rxnLib_ori;
        rxnBlock(currentRxnLength+1:currentRxnLength+length(rxnLib_ori),2) = compartment(i,3);
        rxnBlock(currentRxnLength+1:currentRxnLength+length(rxnLib_ori),3) = compartment(i,4);
        rxnBlock(currentRxnLength+1:currentRxnLength+length(rxnLib_ori),4) = {'INT'};
        
        lbBlock(currentRxnLength+1:currentRxnLength+length(lb_ori)) = lb_ori;
        ubBlock(currentRxnLength+1:currentRxnLength+length(ub_ori)) = ub_ori;
        
        currentSpeLength = size(speciesBlock,1);
        speciesBlock(currentSpeLength+1:currentSpeLength+length(speciesList(:,2)),1) = speciesList(:,2);
        speciesBlock(currentSpeLength+1:currentSpeLength+length(speciesList(:,2)),2) = speciesList(:,1);
        speciesBlock(currentSpeLength+1:currentSpeLength+length(speciesList(:,2)),3) = compartment(i,3);
        speciesBlock(currentSpeLength+1:currentSpeLength+length(speciesList(:,2)),4) = compartment(i,4);
        speciesBlock(currentSpeLength+1:currentSpeLength+length(speciesList(:,2)),5) = {'INT'};
        
        stoicBlock(currentRxnLength+1:currentRxnLength+size(stoicMat,1),currentSpeLength+1:currentSpeLength+size(stoicMat,2)) = stoicMat;
        
    end
end

fprintf('no of mets:\t %1.0f\n',size(stoicBlock,2));
fprintf('no of rxns:\t %1.0f\n',size(stoicBlock,1));
fprintf('\n');
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%put in biomass drain reactions%%%%%
disp('biomass drain reactions addition...');
bioMets = biomassDrain(3:end,2);
bioTag = biomassDrain([1,2],3:end);
bioCoeffCell = biomassDrain(3:end,3:end);
bioCoeff = zeros(size(bioCoeffCell));
for i = 1:size(bioCoeffCell,1)
    for j = 1:size(bioCoeffCell,2)
        if isnumeric(bioCoeffCell{i,j})
            bioCoeff(i,j) = bioCoeffCell{i,j};
        end
    end
end
zeroTags = find(sum(abs(bioCoeff))==0);
bioCoeff(:,zeroTags) = [];
bioTag(:,zeroTags) = [];

currentRxnSize = size(rxnBlock,1);
currentMetSize = size(speciesBlock,1);
for i = 1:size(bioTag,2)
    printCell(bioTag(:,i)');
    rxnBlock{currentRxnSize+i,1} = 'R_BIOMASS';
    rxnBlock{currentRxnSize+i,2} = bioTag{1,i};
    rxnBlock{currentRxnSize+i,3} = bioTag{2,i};
    rxnBlock{currentRxnSize+i,4} = 'BIOM';
    
    speciesBlock{currentMetSize+i,1} = 'S_biomass';
    speciesBlock{currentMetSize+i,2} = 'biomass';
    speciesBlock{currentMetSize+i,3} = bioTag{1,i};
    speciesBlock{currentMetSize+i,4} = bioTag{2,i};
    speciesBlock{currentMetSize+i,5} = 'BIOM';
    
    stoicBlock(currentRxnSize+i,currentMetSize+i) = 1;
    lbBlock(currentRxnSize+i) = 0;
    ubBlock(currentRxnSize+i) = bValue;
    
    for j = 1:size(bioCoeff,1)
        if bioCoeff(j,i) == 0
            continue
        end
        crits = {};
        crits{1} = bioMets{j};
        crits{2} = bioTag{1,i};
        crits{3} = bioTag{2,i};
        crits{4} = 'INT';
        hitMet = fEnt(speciesBlock(:,[1 3 4 5]),crits);
        if length(hitMet) == 0
            fprintf('error, biomass not found: ');
            printCell(crits);
        elseif length(hitMet) > 1
            fprintf('error, biomass multiple match: ');
            printCell(crits);
        else
            stoicBlock(currentRxnSize+i,hitMet) = -bioCoeff(j,i);
        end
    end
end
    
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%put in metabolite connectionss%%%%%
fprintf('\n');
disp('metabolite connections addition...');

currentRxnSize = size(rxnBlock,1);
weightMatrix = cell2mat(compartment(:,[5,6]));
weightMatrix = [weightMatrix weightMatrix(:,1).*weightMatrix(:,2)];
weightMatrix = [weightMatrix weightMatrix(:,3)/sum(weightMatrix(:,3))];
compartmenetTag = compartment(:,[3,4]);

for i = 1:size(metConnections,1)
    printCell(metConnections(i,:));
    species = metConnections{i,2};
    species_long =  speciesList{find(strcmp(species,speciesList(:,2))==1),1};
    speciesSpatial = metConnections{i,3};
    speciesTemporal = metConnections{i,4};
    speciesPoolCNX = metConnections{i,5};
    speciesCNXdirection = metConnections{i,6};
    speciesCNXpenalty = metConnections{i,7};
    
    crits = {};
    crits{1} = speciesSpatial;
    crits{2} = speciesTemporal;
    hitWeight = fEnt(compartmenetTag,crits);
    
    crits = {};
    crits{1} = species;
    crits{2} = speciesPoolCNX;
    hitSharedMet = fEnt(speciesBlock(:,[1 5]),crits);
    
    if length(hitSharedMet) == 0
        speciesBlock{end+1,1} = species;
        speciesBlock{end,2} = species_long;
        speciesBlock{end,5} = speciesPoolCNX;
        hitSharedMet = fEnt(speciesBlock(:,[1 5]),crits);
    end    
    
    crits = {};
    crits{1} = species;
    crits{2} = speciesSpatial;
    crits{3} = speciesTemporal;
    crits{4} = 'INT';
    hitMet = fEnt(speciesBlock(:,[1 3 4 5]),crits);
    
    if length(hitMet) == 0
        fprintf('error, metabolite not found: ');
        printCell(crits);
    elseif length(hitMet) > 1
        fprintf('error, metabolite multiple match: ');
        printCell(crits);
    end
    
    rxnBlock{currentRxnSize+i,1} = strcat(['R_' species]);
    rxnBlock{currentRxnSize+i,2} = speciesSpatial;
    rxnBlock{currentRxnSize+i,3} = speciesTemporal;
    rxnBlock{currentRxnSize+i,4} = speciesPoolCNX;
    
    stoicBlock(currentRxnSize+i,hitMet) = -1;
    if strcmp(speciesPoolCNX,'BOUNDARY')
        stoicBlock(currentRxnSize+i,hitSharedMet) = 1;
    else
        stoicBlock(currentRxnSize+i,hitSharedMet) = weightMatrix(hitWeight,4);
    end
    
    
    if speciesCNXdirection == -1
        lbBlock(currentRxnSize+i) = -bValue;
        ubBlock(currentRxnSize+i) = 0;
    elseif speciesCNXdirection == 1
        lbBlock(currentRxnSize+i) = 0;
        ubBlock(currentRxnSize+i) = bValue;
    else
        lbBlock(currentRxnSize+i) = -bValue;
        ubBlock(currentRxnSize+i) = bValue;
    end
    
    if speciesCNXpenalty > 0
        crits = {};
        
        crits{2} = speciesSpatial;
        crits{3} = speciesTemporal;
        crits{4} = 'INT';
        
        crits{1} = 'S_ATP_c';
        hitATP = fEnt(speciesBlock(:,[1 3 4 5]),crits);
        
        crits{1} = 'S_ADP_c';
        hitADP = fEnt(speciesBlock(:,[1 3 4 5]),crits);
        
        crits{1} = 'S_H2O_c';
        hitH2O = fEnt(speciesBlock(:,[1 3 4 5]),crits);
        
        crits{1} = 'S_Orthophosphate_c';
        hitPI = fEnt(speciesBlock(:,[1 3 4 5]),crits);
        
        if speciesCNXdirection < 0
            stoicBlock(currentRxnSize+i,hitATP) = stoicBlock(currentRxnSize+i,hitATP) + speciesCNXpenalty;
            stoicBlock(currentRxnSize+i,hitH2O) = stoicBlock(currentRxnSize+i,hitH2O) + speciesCNXpenalty;
            stoicBlock(currentRxnSize+i,hitADP) = stoicBlock(currentRxnSize+i,hitADP) +  -speciesCNXpenalty;
            stoicBlock(currentRxnSize+i,hitPI) = stoicBlock(currentRxnSize+i,hitPI) + -speciesCNXpenalty;
        else
            stoicBlock(currentRxnSize+i,hitATP) = stoicBlock(currentRxnSize+i,hitATP) + -speciesCNXpenalty;
            stoicBlock(currentRxnSize+i,hitH2O) = stoicBlock(currentRxnSize+i,hitH2O) + -speciesCNXpenalty;
            stoicBlock(currentRxnSize+i,hitADP) = stoicBlock(currentRxnSize+i,hitADP) + speciesCNXpenalty;
            stoicBlock(currentRxnSize+i,hitPI) = stoicBlock(currentRxnSize+i,hitPI) + speciesCNXpenalty;
        end
    end
end

%%%%put in objective function%%%%%
fprintf('\n');
disp('specifying objective functions')
f = zeros(size(stoicBlock,1),1);

for i = 1:size(objFxn,1)
    printCell(objFxn(i,:));
    crits = {};
    crits{2} = objFxn{i,4};
    crits{3} = objFxn{i,5};
    if strcmp(objFxn{i,2},'bio')
        crits{1} = 'R_BIOMASS';
        crits{4} = 'BIOM';
    elseif strcmp(objFxn{i,2},'met')
        crits{1} = strcat(['R_' objFxn{i,3}]);
        crits{4} = objFxn{i,6};
    else
        crits{1} = objFxn{i,3};
        crits{4} = 'INT';
    end
    hitRxn = fEntMulti(rxnBlock, crits);
    if length(hitRxn) == 0
        fprintf('error, reaction not found: ');
        printCell(crits);
    end
    f(hitRxn) = objFxn{i,7};
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%put in boundary constraints%%%%%
fprintf('\n');
disp('specifying boundary constraints');
for i = 1:size(lbub,1)
    printCell(lbub(i,:));
    
    crits = {};
    crits{2} = lbub{i,4};
    crits{3} = lbub{i,5};
    if strcmp(lbub{i,2},'bio')
        crits{1} = 'R_BIOMASS';
        crits{4} = 'BIOM';
    elseif strcmp(lbub{i,2},'met')
        crits{1} = strcat(['R_' lbub{i,3}]);
        crits{4} = lbub{i,6};
    else
        crits{1} = lbub{i,3};
        crits{4} = 'INT';
    end
    hitRxn = fEntMulti(rxnBlock, crits);
    
    if length(hitRxn) == 0
        fprintf('error, reaction not found: ');
        printCell(crits);
    end
    
    val = lbub{i,7};
    if ischar(val)
        val = str2double(val);
    end
    if isfinite(val)
        lbBlock(hitRxn) = val;
    else
        lbBlock(hitRxn) = -bValue;
    end
    
    val = lbub{i,8};
    if ischar(val)
        val = str2double(val);
    end
    if isfinite(val)
        ubBlock(hitRxn) = val;
    else
        ubBlock(hitRxn) = bValue;
    end
    
end
fprintf('\n');

save modelData stoicBlock f lbBlock ubBlock rxnBlock speciesBlock bValue
disp('model generated')
disp('final model...');
fprintf('no of mets:\t %1.0f\n',size(stoicBlock,2));
fprintf('no of rxns:\t %1.0f\n',size(stoicBlock,1));
fprintf('\n');